/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.threeD;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.text.*;

class ThreeDBorders {
	
	final static class ButtonBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			Insets i1 = ThreeDUtils.getBorderInsets(ThreeDPrefs.buttonBorderType1);
			Insets i2 = ThreeDUtils.getBorderInsets(ThreeDPrefs.buttonBorderType2);
			
			return i1.left > i2.left ? i1 : i2;
			
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			JButton button = (JButton)c;
			ButtonModel model = button.getModel();
			
			if (model.isPressed())
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.buttonBorderType2, g, x, y, width, height);
				}
			else
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.buttonBorderType1, g, x, y, width, height);
				}
		}

	}

	final static class ComboBoxBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			return ThreeDUtils.getBorderInsets(ThreeDPrefs.comboBoxBorderType);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			if (((JComboBox)c).isEditable()) return;
			
			ThreeDUtils.drawBorder(ThreeDPrefs.comboBoxBorderType, g, x, y, width, height);
		}

	}

	final static class ComboBoxPopupBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			return new Insets(0,0,0,0);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
		}

	}

	final static class DesktopIconBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			return ThreeDUtils.getBorderInsets(ThreeDPrefs.buttonBorderType1);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			ThreeDUtils.drawBorder(ThreeDPrefs.buttonBorderType1, g, x, y, width, height);
		}

	}

	final static class InternalFrameBorder extends AbstractBorder {
		static final int SIZER = 15;
		
		public Insets getBorderInsets(Component c)  {
			return new Insets(4,4,4,4);
			
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			g.translate(x,y);
			
			if (((JInternalFrame)c).isSelected())
				{
				g.setColor(ThreeDPrefs.highlightColor);
				g.fillRect(0,0,width,4);
				g.fillRect(0,0,4,height);
				g.fillRect(width-5,0,4,height);
				g.fillRect(0,height-5,width,4);
				}
			
			ThreeDUtils.drawBorder(ThreeDUtils.RAISED_THIN, g, 0, 0, width, height);
			ThreeDUtils.drawBorder(ThreeDUtils.LOWERED_THIN, g, 3, 3, width-6, height-6);
			
			if (((JInternalFrame)c).isResizable())
				{
				// Oben
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(SIZER,0,SIZER,3);
				g.drawLine(width-1-SIZER,0,width-1-SIZER,3);
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(SIZER+1,0,SIZER+1,3);
				g.drawLine(width-SIZER,0,width-SIZER,3);
				// Unten
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(SIZER,height-1,SIZER,height-4);
				g.drawLine(width-1-SIZER,height-1,width-1-SIZER,height-4);
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(SIZER+1,height-1,SIZER+1,height-4);
				g.drawLine(width-SIZER,height-1,width-SIZER,height-4);
				// Links
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(0,SIZER,3,SIZER);
				g.drawLine(0,height-1-SIZER,3,height-1-SIZER);
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(0,SIZER+1,3,SIZER+1);
				g.drawLine(0,height-SIZER,3,height-SIZER);
				// Rechts
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(width-1,SIZER,width-4,SIZER);
				g.drawLine(width-1,height-1-SIZER,width-4,height-1-SIZER);
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(width-1,SIZER+1,width-4,SIZER+1);
				g.drawLine(width-1,height-SIZER,width-4,height-SIZER);
				}
			
			g.translate(-x,-y);
		}

	}

	final static class ListBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			if (c.getParent() instanceof JViewport) return new Insets(0,0,0,0);
			return ThreeDUtils.getBorderInsets(ThreeDPrefs.listBorderType);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			if (c.getParent() instanceof JViewport) return;
			ThreeDUtils.drawBorder(ThreeDPrefs.listBorderType, g, x, y, width, height);
		}

	}

	final static class ListFocusCellHighlightBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			return new Insets(1,1,1,1);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			g.setColor(ThreeDPrefs.focusColor);
			g.drawRect(x,y,width-1,height-1);
		}

	}

	final static class MenuBarBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.menuBarBorderType);
			return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			ThreeDUtils.drawBorder(ThreeDPrefs.menuBarBorderType, g, x, y, width, height);
		}

	}

	final static class MenuBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.menuBorderType);
			return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			JMenu b = (JMenu) c;
			ButtonModel model = b.getModel();
			
			if (model.isSelected())
				{
				if (b.getParent() instanceof JMenuBar)
					{
					int add = ThreeDUtils.isLoweredBorderType(ThreeDPrefs.menuBorderType) ? 10 : 0;
					ThreeDUtils.drawBorder(ThreeDPrefs.menuBorderType, g, x, y, width, height+add);
					}
				else // Untermenu
					{
					int add = ThreeDUtils.isLoweredBorderType(ThreeDPrefs.menuBorderType) ? 10 : 0;
					ThreeDUtils.drawBorder(ThreeDPrefs.menuBorderType, g, x, y, width+add, height);
					}
				}
		}

	}

	final static class MenuItemBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			return ThreeDUtils.getBorderInsets(ThreeDPrefs.menuItemBorderType);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			JMenuItem b = (JMenuItem) c;
			ButtonModel model = b.getModel();
			
			if (model.isArmed())
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.menuItemBorderType, g, x, y, width, height);
				}
		}

	}

	final static class PopupMenuBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			if (c instanceof ThreeD_ComboBoxPopup) return new Insets(0,0,0,0);
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.popupMenuBorderType);
			return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			if (c instanceof ThreeD_ComboBoxPopup) return;
			ThreeDUtils.drawBorder(ThreeDPrefs.popupMenuBorderType, g, x, y, width, height);
		}

	}

	final static class ProgressBarBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			return ThreeDUtils.getBorderInsets(ThreeDPrefs.progressBarBorderType);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			ThreeDUtils.drawBorder(ThreeDPrefs.progressBarBorderType, g, x, y, width, height);
		}

	}

	final static class ScrollPaneBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			return new Insets(0,0,0,0);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
		}

	}

	final static class ScrollPaneViewportBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			Component view = ((JScrollPane)c).getViewport().getView();
			
			if (view != null && view instanceof JTree)
				{
				Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.treeBorderType);
				return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
				}
			else if (view != null && view instanceof JTable)
				{
				Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.tableBorderType);
				return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
				}
			else if (view != null && view instanceof JList)
				{
				Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.listBorderType);
				return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
				}
			else if (view != null && (view instanceof JEditorPane || view instanceof JTextArea) )
				{
				Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.textComponentBorderType1);
				if (! ((JTextComponent)view).isEditable()) i = ThreeDUtils.getBorderInsets(ThreeDPrefs.textComponentBorderType2);
				return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
				}
			else return new Insets(0,0,0,0);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			Component view = ((JScrollPane)c).getViewport().getView();
			
			if (view != null && view instanceof JTree)
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.treeBorderType, g, x, y, width, height);
				}
			else if (view != null && view instanceof JTable)
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.tableBorderType, g, x, y, width, height);
				}
			else if (view != null && view instanceof JList)
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.listBorderType, g, x, y, width, height);
				}
			else if (view != null && (view instanceof JEditorPane || view instanceof JTextArea) )
				{
				JTextComponent tc = (JTextComponent) view;
				if (tc.isEditable())
					ThreeDUtils.drawBorder(ThreeDPrefs.textComponentBorderType1, g, x, y, width, height);
				else
					ThreeDUtils.drawBorder(ThreeDPrefs.textComponentBorderType2, g, x, y, width, height);
				}
		}

	}

	final static class SplitPaneBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			return new Insets(0,0,0,0);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
		}

	}

	final static class TableBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			return new Insets(2,2,2,2);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
		}

	}

	final static class TableHeaderBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.tableHeaderBorderType);
			return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			ThreeDUtils.drawBorder(ThreeDPrefs.tableHeaderBorderType, g, x, y, width, height);
		}

	}

	final static class TextComponentBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			if (c.getParent() instanceof JViewport) return new Insets(0,0,0,0);
			return ((JTextComponent)c).isEditable() 
				?	ThreeDUtils.getBorderInsets(ThreeDPrefs.textComponentBorderType1) 
				: 	ThreeDUtils.getBorderInsets(ThreeDPrefs.textComponentBorderType2);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			if (c.getParent() instanceof JViewport) return;
			
			JTextComponent tc = (JTextComponent)c;
			
			if (tc.getParent() instanceof JViewport)
				{
				Rectangle r = ((JViewport)tc.getParent()).getViewRect();
				x = r.x;
				y = r.y;
				width = r.width;
				height = r.height;
				}
			
			if (tc.isEditable())
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.textComponentBorderType1, g, x, y, width, height);
				}
			else 
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.textComponentBorderType2, g, x, y, width, height);
				}
		}

	}

	final static class TitledBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			return new Insets(2,2,2,2);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			g.setColor(ThreeDPrefs.darkEdgeColor);
			g.drawRect(x,y,width-2,height-2);
			g.setColor(ThreeDPrefs.lightEdgeColor);
			g.drawRect(x+1,y+1,width-2,height-2);
		}

	}

	final static class ToggleButtonBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			Insets i1 = ThreeDUtils.getBorderInsets(ThreeDPrefs.toggleButtonBorderType1);
			Insets i2 = ThreeDUtils.getBorderInsets(ThreeDPrefs.toggleButtonBorderType2);
			
			return i1.left > i2.left ? i1 : i2;
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			JToggleButton button = (JToggleButton)c;
			ButtonModel model = button.getModel();
			
			if (model.isPressed() || model.isSelected())
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.toggleButtonBorderType2, g, x, y, width, height);
				}
			else
				{
				ThreeDUtils.drawBorder(ThreeDPrefs.toggleButtonBorderType1, g, x, y, width, height);
				}
		}

	}

	final static class ToolBarBorder extends AbstractBorder implements UIResource {
		
		public Insets getBorderInsets(Component c)  {
			JToolBar 	tb 				= (JToolBar) c;
			Insets 		borderInsets 	= new Insets(4,4,4,4);
			
			if (tb.isFloatable())
				{
				if (tb.getOrientation() == JToolBar.HORIZONTAL )
					{
					borderInsets.left = 16;
					}
				else // vertical
					{
					borderInsets.top = 16;
					}
				}
			
			Insets margin = tb.getMargin();
			if (margin != null)
				{
				borderInsets.left		+= margin.left;
				borderInsets.top		+= margin.top;
				borderInsets.right	+= margin.right;
				borderInsets.bottom	+= margin.bottom;
				}
			
			return borderInsets;
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
			JToolBar 	tb 	= (JToolBar)c;
			Dimension	size	= tb.getSize();
			Insets		ins	= tb.getInsets();
			
			g.translate(x, y);
			
			if (tb.isFloatable() )
				{
				if (tb.getOrientation() == JToolBar.HORIZONTAL )
					{
					g.setColor(ThreeDPrefs.highlightColor);
					g.fillRect(3,3,8,size.height-5);
					ThreeDUtils.drawBorder(g, 2, 2, 10, size.height-4, ThreeDPrefs.lightEdgeColor, ThreeDPrefs.darkEdgeColor);
					}
				else // vertical
					{
					g.setColor(ThreeDPrefs.highlightColor);
					g.fillRect(3,3,size.width-5,8);
					ThreeDUtils.drawBorder(g, 2, 2, size.width-4, 10, ThreeDPrefs.lightEdgeColor, ThreeDPrefs.darkEdgeColor);
					}
				}
			
			g.translate(-x, -y);
			   
		}

	}

}
