/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.threeD;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;

class ThreeDPrefsFrame extends JFrame {
	// Komponenten
	JButton 		save 						= new JButton("Save");
	JButton 		test						= new JButton("Test");
	JButton 		cancel						= new JButton("Cancel");
	JFileChooser	fileChooser					= new JFileChooser(); 
	
	// System
	JFontSelector	font1						= new JFontSelector();
	JColorSelector	backgroundColor			= new JColorSelector();
	JColorSelector	textColor					= new JColorSelector();
	JColorSelector	highlightColor				= new JColorSelector();
	JColorSelector	focusColor					= new JColorSelector();
	JColorSelector	disableColor				= new JColorSelector();
	JColorSelector	disabledTextColor			= new JColorSelector();
	
	// Borders
	JColorSelector		lightEdgeColor					= new JColorSelector();
	JColorSelector		darkEdgeColor					= new JColorSelector();
	JBorderSelector	buttonBorderType1				= new JBorderSelector();
	JBorderSelector	buttonBorderType2				= new JBorderSelector();
	JBorderSelector	checkBoxBorderType1			= new JBorderSelector();
	JBorderSelector	checkBoxBorderType2			= new JBorderSelector();
	JBorderSelector	comboBoxBorderType			= new JBorderSelector();
	JBorderSelector	listBorderType					= new JBorderSelector();
	JBorderSelector	menuBorderType				= new JBorderSelector();
	JBorderSelector	menuBarBorderType			= new JBorderSelector();
	JBorderSelector	menuItemBorderType			= new JBorderSelector();
	JBorderSelector	popupMenuBorderType			= new JBorderSelector();
	JBorderSelector	progressBarBorderType			= new JBorderSelector();
	JBorderSelector	radioButtonBorderType1		= new JBorderSelector();
	JBorderSelector	radioButtonBorderType2		= new JBorderSelector();
	JBorderSelector	scrollBarThumbBorderType		= new JBorderSelector();
	JBorderSelector	scrollBarTrackBorderType		= new JBorderSelector();
	JBorderSelector	sliderThumbBorderType			= new JBorderSelector();
	JBorderSelector	sliderTrackBorderType			= new JBorderSelector();
	JBorderSelector	tableBorderType				= new JBorderSelector();
	JBorderSelector	tableHeaderBorderType		= new JBorderSelector();
	JBorderSelector	textComponentBorderType1	= new JBorderSelector();
	JBorderSelector	textComponentBorderType2	= new JBorderSelector();
	JBorderSelector	toggleButtonBorderType1		= new JBorderSelector();
	JBorderSelector	toggleButtonBorderType2		= new JBorderSelector();
	JBorderSelector	treeBorderType					= new JBorderSelector();
	
	// Desktop
	JColorSelector	desktopColor				= new JColorSelector();
	
	// Menus
	JColorSelector	menuBackgroundColor		= new JColorSelector();
	JColorSelector	menuTextColor				= new JColorSelector();
	JColorSelector	menuHighlightColor			= new JColorSelector();
	JColorSelector	disabledMenuTextColor		= new JColorSelector();
	JFontSelector	font2						= new JFontSelector();
	JColorSelector	acceleratorColor			= new JColorSelector();
	
	// ScrollBar
	JColorSelector	scrollBarTrackColor			= new JColorSelector();
	JColorSelector	scrollBarThumbColor		= new JColorSelector();
	
	// Slider
	JColorSelector	sliderTrackColor			= new JColorSelector();
	JColorSelector	sliderThumbColor			= new JColorSelector();
	
	// TabbedPane
	JColorSelector	tabbedPaneBackgroundColor	= new JColorSelector();
	
	// TextComponents
	JColorSelector	textFieldBackgroundColor	= new JColorSelector();
	JColorSelector	caretColor					= new JColorSelector();
	JComboBox	caretBlinkRate					= new JComboBox(new String [] { "0", "200", "400", "600", "800", "1000" });
	
	// TitledBorders
	JFontSelector	font3						= new JFontSelector();
	JColorSelector	titledBorderTextColor		= new JColorSelector();
	
	// ToolTips
	JColorSelector	toolTipBackgroundColor		= new JColorSelector();
	
	public ThreeDPrefsFrame()  {
		super("Prefs for 3D Look & Feel");
		
		// Panel erzeugen
		JPanel p =(JPanel) getContentPane();
		p.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.add(createPrefsPanel());
		p.add(new Strut());
		p.add(new JPopupMenu.Separator());
		p.add(new Strut());
		p.add(createButtons());
		
		// WindowListener
		addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { System.exit(0); }});
		
		// Menus
		setJMenuBar(createMenuBar());
		
		// Komponenten einrichten
		updateComponents();
		
		// Anzeigen
		pack();
		Dimension size = getSize();
		if (size.width < 500) { size.width = 500; setSize(size); }
		setLocation(100,100);
		setVisible(true);
	}

	private void about()  {
		JOptionPane.showMessageDialog(this, UIManager.getLookAndFeel().getDescription());
	}

	private JPanel createButtons()  {
		JPanel result = new JPanel();
		
		result.setLayout(new BoxLayout(result, BoxLayout.X_AXIS));
		result.add(save);
		result.add(Box.createHorizontalGlue());
		result.add(test);
		result.add(Box.createHorizontalGlue());
		result.add(cancel);
		
		save.addActionListener(new ActionListener() { 
			public void actionPerformed(ActionEvent e) { 
				setVisible(false);
				test(); 
				ThreeDPrefs.save(); 
				System.exit(0);
			}
		});
		
		test.addActionListener(new ActionListener() { 
			public void actionPerformed(ActionEvent e) { 
				test(); 
			}
		});
		
		cancel.addActionListener(new ActionListener() { 
			public void actionPerformed(ActionEvent e) { 
				System.exit(0);
			}
		});
		
		return result;
	}

	private JMenuBar createMenuBar()  {
		JMenuBar 	result 	= new JMenuBar();
		JMenu 		menu	= null;
		JMenuItem	item	= null;
		
		// Program
		menu = result.add(new JMenu("Program"));
		item = menu.add(new JMenuItem("Open ..."));
		item.setAccelerator(KeyStroke.getKeyStroke("control O"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { load(); }});
		item = menu.add(new JMenuItem("Save as ..."));
		item.setAccelerator(KeyStroke.getKeyStroke("control S"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { saveAs(); }});
		menu.addSeparator();
		item = menu.add(new JMenuItem("About"));
		item.setAccelerator(KeyStroke.getKeyStroke("control A"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { about(); }});
		menu.addSeparator();
		item = menu.add(new JMenuItem("Quit"));
		item.setAccelerator(KeyStroke.getKeyStroke("control Q"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { System.exit(0); }});
		// Edit
		menu = result.add(new JMenu("Edit"));
		item = menu.add(new JMenuItem("Reset to Default Values"));
		item.setAccelerator(KeyStroke.getKeyStroke("control D"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { defaultValues(); }});
		item = menu.add(new JMenuItem("Reset to Last Saved"));
		item.setAccelerator(KeyStroke.getKeyStroke("control L"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { lastSaved(); }});
		
		return result;
	}

	private JPanel createPanel_Borders()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Light Edge Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(lightEdgeColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Dark Edge Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(darkEdgeColor, c);
		
		c.gridx=0; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Button (1):"), c);
		c.gridx=1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(buttonBorderType1, c);
		
		c.gridx=0; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Button (2):"), c);
		c.gridx=1; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(buttonBorderType2, c);
		
		c.gridx=0; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("CheckBox (1):"), c);
		c.gridx=1; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(checkBoxBorderType1, c);
		
		c.gridx=0; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("CheckBox (2):"), c);
		c.gridx=1; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(checkBoxBorderType2, c);
		
		c.gridx=0; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ComboBox:"), c);
		c.gridx=1; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(comboBoxBorderType, c);
		
		c.gridx=0; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("List:"), c);
		c.gridx=1; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(listBorderType, c);
		
		// Zweite Spalte
		
		c.gridx=2; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("MenuBar:"), c);
		c.gridx=3; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuBarBorderType, c);
		
		c.gridx=2; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Menu:"), c);
		c.gridx=3; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuBorderType, c);
		
		c.gridx=2; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("MenuItem:"), c);
		c.gridx=3; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuItemBorderType, c);
		
		c.gridx=2; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("PopupMenu:"), c);
		c.gridx=3; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(popupMenuBorderType, c);
		
		c.gridx=2; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ProgressBar:"), c);
		c.gridx=3; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(progressBarBorderType, c);
		
		c.gridx=2; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("RadioButton (1):"), c);
		c.gridx=3; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(radioButtonBorderType1, c);
		
		c.gridx=2; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("RadioButton (2):"), c);
		c.gridx=3; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(radioButtonBorderType2, c);
		
		c.gridx=2; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ScrollBar Thumb:"), c);
		c.gridx=3; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(scrollBarThumbBorderType, c);
		
		// Dritte Reihe
		
		c.gridx=4; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ScrollBar Track:"), c);
		c.gridx=5; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(scrollBarTrackBorderType, c);
		
		c.gridx=4; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Slider Thumb:"), c);
		c.gridx=5; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(sliderThumbBorderType, c);
		
		c.gridx=4; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Slider Track:"), c);
		c.gridx=5; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(sliderTrackBorderType, c);
		
		c.gridx=4; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Table:"), c);
		c.gridx=5; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(tableBorderType, c);
		
		c.gridx=4; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Table Header:"), c);
		c.gridx=5; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(tableHeaderBorderType, c);
		
		c.gridx=4; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("TextComponent (1):"), c);
		c.gridx=5; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(textComponentBorderType1, c);
		
		c.gridx=4; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("TextComponent (2):"), c);
		c.gridx=5; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(textComponentBorderType2, c);
		
		c.gridx=4; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ToggleButton (1):"), c);
		c.gridx=5; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(toggleButtonBorderType1, c);
		
		// Vierte Zeile
		
		c.gridx=6; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ToggleButton (2):"), c);
		c.gridx=7; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(toggleButtonBorderType2, c);
		
		c.gridx=6; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Tree:"), c);
		c.gridx=7; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(treeBorderType, c);
		
		return result;
	}

	private JPanel createPanel_Desktop()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Desktop Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(desktopColor, c);
		
		return result;
	}

	private JPanel createPanel_Menus()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Background Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuBackgroundColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Text Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuTextColor, c);
		
		c.gridx=0; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Highlight Color:"), c);
		c.gridx=1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuHighlightColor, c);
		
		c.gridx=0; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Disabled Text Color:"), c);
		c.gridx=1; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(disabledMenuTextColor, c);
		
		c.gridx=0; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Accelerator Font:"), c);
		c.gridx=1; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(font2, c);
		
		c.gridx=0; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Accelerator Color:"), c);
		c.gridx=1; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(acceleratorColor, c);
		
		return result;
	}

	private JPanel createPanel_ScrollBar()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Track Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(scrollBarTrackColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Thumb Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(scrollBarThumbColor, c);
		
		return result;
	}

	private JPanel createPanel_Slider()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Track Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(sliderTrackColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Thumb Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(sliderThumbColor, c);
		
		return result;
	}

	private JPanel createPanel_System()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Font for Components:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(font1, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Background Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(backgroundColor, c);
		
		c.gridx=0; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Text Color:"), c);
		c.gridx=1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(textColor, c);
		
		c.gridx=0; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Highlight Color:"), c);
		c.gridx=1; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(highlightColor, c);
		
		c.gridx=0; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Focus Color:"), c);
		c.gridx=1; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(focusColor, c);
		
		c.gridx=0; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Disable Color:"), c);
		c.gridx=1; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(disableColor, c);
		
		c.gridx=0; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Disabled Text Color:"), c);
		c.gridx=1; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(disabledTextColor, c);
		
		return result;
	}

	private JPanel createPanel_TabbedPane()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Color for unselected Tabs:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(tabbedPaneBackgroundColor, c);
		
		return result;
	}

	private JPanel createPanel_TextComponents()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Background Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(textFieldBackgroundColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Caret Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(caretColor, c);
		
		caretBlinkRate.setEditable(true);
		c.gridx=0; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Caret Blink Rate:"), c);
		c.gridx=1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(caretBlinkRate, c);
		
		return result;
	}

	private JPanel createPanel_TitledBorder()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Titled Border Font:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(font3, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Titled Border Text Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(titledBorderTextColor, c);
		
		return result;
	}

	private JPanel createPanel_ToolTips()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Background Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(toolTipBackgroundColor, c);
		
		return result;
	}

	private JTabbedPane createPrefsPanel()  {
		JTabbedPane result = new JTabbedPane();
		
		result.addTab("System"				, createPanel_System());
		result.addTab("Borders"				, createPanel_Borders());
		result.addTab("Desktops"			, createPanel_Desktop());
		result.addTab("Menus"				, createPanel_Menus());
		result.addTab("ScrollBars"			, createPanel_ScrollBar());
		result.addTab("Sliders"				, createPanel_Slider());
		result.addTab("TabbedPanes"		, createPanel_TabbedPane());
		result.addTab("TextComponents"		, createPanel_TextComponents());
		result.addTab("TitledBorders"			, createPanel_TitledBorder());
		result.addTab("ToolTips"			, createPanel_ToolTips());
		
		return result;
	}

	private void defaultValues()  {
		ThreeDPrefs.resetToDefaultValues();
		updateComponents();
		updateLookAndFeel();
	}

	private void lastSaved()  {
		try
			{
			ThreeDPrefs.resetToLastSaved();
			}
		catch (IOException x) { ThreeDPrefs.resetToDefaultValues(); }
		
		updateComponents();
		updateLookAndFeel();
	}

	private void load()  {
		if (fileChooser.showOpenDialog(this) == JFileChooser.CANCEL_OPTION) return;
		
		File f = fileChooser.getSelectedFile();
		if (f == null) return;
		if (f.isDirectory()) return;
		
		try
			{
			ThreeDPrefs.load(f.getAbsolutePath());
			}
		catch (Exception x) 
			{ 
			JOptionPane.showMessageDialog(this, x, "Exception caught:", JOptionPane.ERROR_MESSAGE); 
			ThreeDPrefs.resetToDefaultValues();
			}
		
		updateComponents();
		updateLookAndFeel();
	}

	public static void main(String [] args)  {
		try 
			{
			UIManager.installLookAndFeel("3D Look & Feel", "swing.addon.plaf.threeD.ThreeDLookAndFeel");
			UIManager.setLookAndFeel("swing.addon.plaf.threeD.ThreeDLookAndFeel");
			} 
		catch (Exception exc) 
			{
			System.err.println("Error loading L&F: " + exc);
			}
		
		new ThreeDPrefsFrame();
	}

	private void saveAs()  {
		if (fileChooser.showSaveDialog(this) == JFileChooser.CANCEL_OPTION) return;
		
		File f = fileChooser.getSelectedFile();
		if (f == null) return;
		if (f.isDirectory()) return;
		
		try
			{
			ThreeDPrefs.save(f.getAbsolutePath());
			}
		catch (Exception x) { JOptionPane.showMessageDialog(this, x, "Exception caught:", JOptionPane.ERROR_MESSAGE); }
	}

	private void test()  {
		// System
		ThreeDPrefs.normalFont 					= font1.getFontUIResource();
		ThreeDPrefs.backgroundColor			= backgroundColor.getColor();
		ThreeDPrefs.textColor					= textColor.getColor();
		ThreeDPrefs.highlightColor				= highlightColor.getColor();
		ThreeDPrefs.focusColor					= focusColor.getColor();
		ThreeDPrefs.disableColor				= disableColor.getColor();
		ThreeDPrefs.disabledTextColor			= disabledTextColor.getColor();
		
		// Borders
		ThreeDPrefs.lightEdgeColor				= lightEdgeColor.getColor();
		ThreeDPrefs.darkEdgeColor				= darkEdgeColor.getColor();
		ThreeDPrefs.buttonBorderType1			= buttonBorderType1.getBorderType();
		ThreeDPrefs.buttonBorderType2			= buttonBorderType2.getBorderType();
		ThreeDPrefs.checkBoxBorderType1		= checkBoxBorderType1.getBorderType();
		ThreeDPrefs.checkBoxBorderType2		= checkBoxBorderType2.getBorderType();
		ThreeDPrefs.comboBoxBorderType		= comboBoxBorderType.getBorderType();
		ThreeDPrefs.listBorderType				= listBorderType.getBorderType();
		ThreeDPrefs.menuBorderType			= menuBorderType.getBorderType();
		ThreeDPrefs.menuBarBorderType			= menuBarBorderType.getBorderType();
		ThreeDPrefs.menuItemBorderType		= menuItemBorderType.getBorderType();
		ThreeDPrefs.popupMenuBorderType		= popupMenuBorderType.getBorderType();
		ThreeDPrefs.progressBarBorderType		= progressBarBorderType.getBorderType();
		ThreeDPrefs.radioButtonBorderType1		= radioButtonBorderType1.getBorderType();
		ThreeDPrefs.radioButtonBorderType2		= radioButtonBorderType2.getBorderType();
		ThreeDPrefs.scrollBarThumbBorderType	= scrollBarThumbBorderType.getBorderType();
		ThreeDPrefs.scrollBarTrackBorderType	= scrollBarTrackBorderType.getBorderType();
		ThreeDPrefs.sliderThumbBorderType		= sliderThumbBorderType.getBorderType();
		ThreeDPrefs.sliderTrackBorderType		= sliderTrackBorderType.getBorderType();
		ThreeDPrefs.tableBorderType				= tableBorderType.getBorderType();
		ThreeDPrefs.tableHeaderBorderType		= tableHeaderBorderType.getBorderType();
		ThreeDPrefs.textComponentBorderType1	= textComponentBorderType1.getBorderType();
		ThreeDPrefs.textComponentBorderType2	= textComponentBorderType2.getBorderType();
		ThreeDPrefs.toggleButtonBorderType1	= toggleButtonBorderType1.getBorderType();
		ThreeDPrefs.toggleButtonBorderType2	= toggleButtonBorderType2.getBorderType();
		ThreeDPrefs.treeBorderType				= treeBorderType.getBorderType();
		
		// Desktop
		ThreeDPrefs.desktopColor				= desktopColor.getColor();
		
		// Menus
		ThreeDPrefs.menuBackgroundColor		= menuBackgroundColor.getColor();
		ThreeDPrefs.menuTextColor				= menuTextColor.getColor();
		ThreeDPrefs.menuHighlightColor			= menuHighlightColor.getColor();
		ThreeDPrefs.disabledMenuTextColor		= disabledMenuTextColor.getColor();
		ThreeDPrefs.acceleratorFont 				= font2.getFontUIResource();
		ThreeDPrefs.acceleratorColor				= acceleratorColor.getColor();
		
		// ScrollBar
		ThreeDPrefs.scrollBarTrackColor			= scrollBarTrackColor.getColor();
		ThreeDPrefs.scrollBarThumbColor			= scrollBarThumbColor.getColor();
		
		// Slider
		ThreeDPrefs.sliderTrackColor				= sliderTrackColor.getColor();
		ThreeDPrefs.sliderThumbColor			= sliderThumbColor.getColor();
		
		// TabbedPane
		ThreeDPrefs.tabbedPaneBackgroundColor	= tabbedPaneBackgroundColor.getColor();
		
		// TextComponents
		ThreeDPrefs.textFieldBackgroundColor	= textFieldBackgroundColor.getColor();
		ThreeDPrefs.caretColor					= caretColor.getColor();
		ThreeDPrefs.caretBlinkRate				= Integer.parseInt((String)caretBlinkRate.getSelectedItem());
		
		// TitledBorders
		ThreeDPrefs.titledBorderFont 			= font3.getFontUIResource();
		ThreeDPrefs.titledBorderTextColor		= titledBorderTextColor.getColor();
		
		// ToolTips
		ThreeDPrefs.toolTipBackgroundColor		= toolTipBackgroundColor.getColor();
		
		// Setzen
		updateLookAndFeel();
	}

	private void updateComponents()  {
		// System
		font1.setUserFont(ThreeDPrefs.normalFont);
		backgroundColor.setColor(ThreeDPrefs.backgroundColor);
		textColor.setColor(ThreeDPrefs.textColor);
		highlightColor.setColor(ThreeDPrefs.highlightColor);
		focusColor.setColor(ThreeDPrefs.focusColor);
		disableColor.setColor(ThreeDPrefs.disableColor);
		disabledTextColor.setColor(ThreeDPrefs.disabledTextColor);
		
		// Borders
		lightEdgeColor.setColor(ThreeDPrefs.lightEdgeColor);
		darkEdgeColor.setColor(ThreeDPrefs.darkEdgeColor);
		buttonBorderType1.setBorderType(ThreeDPrefs.buttonBorderType1);
		buttonBorderType2.setBorderType(ThreeDPrefs.buttonBorderType2);
		checkBoxBorderType1.setBorderType(ThreeDPrefs.checkBoxBorderType1);
		checkBoxBorderType2.setBorderType(ThreeDPrefs.checkBoxBorderType2);
		comboBoxBorderType.setBorderType(ThreeDPrefs.comboBoxBorderType);
		listBorderType.setBorderType(ThreeDPrefs.listBorderType);
		menuBorderType.setBorderType(ThreeDPrefs.menuBorderType);
		menuBarBorderType.setBorderType(ThreeDPrefs.menuBarBorderType);
		menuItemBorderType.setBorderType(ThreeDPrefs.menuItemBorderType);
		popupMenuBorderType.setBorderType(ThreeDPrefs.popupMenuBorderType);
		progressBarBorderType.setBorderType(ThreeDPrefs.progressBarBorderType);
		radioButtonBorderType1.setBorderType(ThreeDPrefs.radioButtonBorderType1);
		radioButtonBorderType2.setBorderType(ThreeDPrefs.radioButtonBorderType2);
		scrollBarThumbBorderType.setBorderType(ThreeDPrefs.scrollBarThumbBorderType);
		scrollBarTrackBorderType.setBorderType(ThreeDPrefs.scrollBarTrackBorderType);
		sliderThumbBorderType.setBorderType(ThreeDPrefs.sliderThumbBorderType);
		sliderTrackBorderType.setBorderType(ThreeDPrefs.sliderTrackBorderType);
		tableBorderType.setBorderType(ThreeDPrefs.tableBorderType);
		tableHeaderBorderType.setBorderType(ThreeDPrefs.tableHeaderBorderType);
		textComponentBorderType1.setBorderType(ThreeDPrefs.textComponentBorderType1);
		textComponentBorderType2.setBorderType(ThreeDPrefs.textComponentBorderType2);
		toggleButtonBorderType1.setBorderType(ThreeDPrefs.toggleButtonBorderType1);
		toggleButtonBorderType2.setBorderType(ThreeDPrefs.toggleButtonBorderType2);
		treeBorderType.setBorderType(ThreeDPrefs.treeBorderType);
		
		// Desktop
		desktopColor.setColor(ThreeDPrefs.desktopColor);
		
		// Menus
		menuBackgroundColor.setColor(ThreeDPrefs.menuBackgroundColor);
		menuTextColor.setColor(ThreeDPrefs.menuTextColor);
		menuHighlightColor.setColor(ThreeDPrefs.menuHighlightColor);
		disabledMenuTextColor.setColor(ThreeDPrefs.disabledMenuTextColor);
		font2.setUserFont(ThreeDPrefs.acceleratorFont);
		acceleratorColor.setColor(ThreeDPrefs.acceleratorColor);
		
		// Slider
		scrollBarTrackColor	.setColor(ThreeDPrefs.scrollBarTrackColor);
		scrollBarThumbColor.setColor(ThreeDPrefs.scrollBarThumbColor);
		
		// Slider
		sliderTrackColor	.setColor(ThreeDPrefs.sliderTrackColor);
		sliderThumbColor.setColor(ThreeDPrefs.sliderThumbColor);
		
		// TabbedPane
		tabbedPaneBackgroundColor.setColor(ThreeDPrefs.tabbedPaneBackgroundColor);
		
		// TextComponents
		textFieldBackgroundColor.setColor(ThreeDPrefs.textFieldBackgroundColor);
		caretColor.setColor(ThreeDPrefs.caretColor);
		caretBlinkRate.setSelectedItem(""+ThreeDPrefs.caretBlinkRate);
		caretBlinkRate.getEditor().setItem(""+ThreeDPrefs.caretBlinkRate);
		
		// TitledBorders
		font3.setUserFont(ThreeDPrefs.titledBorderFont);
		titledBorderTextColor.setColor(ThreeDPrefs.titledBorderTextColor);
		
		// ToolTips
		toolTipBackgroundColor.setColor(ThreeDPrefs.toolTipBackgroundColor);
	}

	private void updateLookAndFeel()  {
		try 
			{
			UIManager.setLookAndFeel("swing.addon.plaf.threeD.ThreeDLookAndFeel");
			SwingUtilities.updateComponentTreeUI(SwingUtilities.getRoot(this));
			Dimension preferred = getPreferredSize();
			Dimension size = getSize();
			if (preferred.width > size.width) size.width = preferred.width;
			if (preferred.height > size.height) size.height = preferred.height;
			setSize(size);
			} 
		catch (Exception exc) { System.err.println(exc); }
	}

}
