/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.threeD;

import java.awt.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class ThreeD_SliderUI extends BasicSliderUI {
	
	public ThreeD_SliderUI(JSlider s)  {
		super(s);
	}

	public static ComponentUI createUI(JComponent c)  {
		return new ThreeD_SliderUI((JSlider)c);
	}

	public void paint(Graphics g, JComponent c)  {
		super.paint(g, c);
		
		if (! c.isEnabled()) ThreeDUtils.drawDisabledState(g, c);
	}

	public void paintFocus(Graphics g)  {
		Dimension size = slider.getSize();
		
		g.setColor(ThreeDPrefs.focusColor);
		g.drawRect(0,0,size.width-1,size.height-1);
	}

	public void paintThumb(Graphics g)  {
		//Hintergrund malen
		g.setColor(slider.getBackground());
		g.fillRect(thumbRect.x, thumbRect.y, thumbRect.width, thumbRect.height);
		
		// Rahmen malen
		ThreeDUtils.drawBorder(ThreeDPrefs.sliderThumbBorderType, g, thumbRect.x, thumbRect.y, thumbRect.width, thumbRect.height);
		
		// farbig ausmalen
		Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.sliderThumbBorderType);
		g.setColor(ThreeDPrefs.sliderThumbColor);
		g.fillRect(thumbRect.x+i.left, thumbRect.y+i.top, thumbRect.width-i.left-i.right, thumbRect.height-i.top-i.bottom);
	}

	public void paintTrack(Graphics g)  {
		// Hintergrund lschen
		g.setColor(slider.getBackground());
		g.fillRect(trackRect.x, trackRect.y, trackRect.width-1, trackRect.height-1);
		
		// Insets holen
		Insets i = ThreeDUtils.getBorderInsets(ThreeDPrefs.sliderTrackBorderType);
		
		// berechnen
		Rectangle draw = new Rectangle(trackRect.x, trackRect.y, trackRect.width, trackRect.height);
		if (slider.getOrientation() == JSlider.HORIZONTAL)
			{
			draw.height = trackRect.height/3 + trackRect.height % 3 + i.top + i.bottom;
			draw.y = draw.y + (trackRect.height - draw.height)/2;
			}
		else
			{
			draw.width = trackRect.width/3 + trackRect.width % 3 + i.left + i.right;
			draw.x = draw.x + (trackRect.width - draw.width)/2;
			}
		
		// Rahmen malen
		ThreeDUtils.drawBorder(ThreeDPrefs.sliderTrackBorderType, g, draw.x, draw.y, draw.width, draw.height);
		
		// Vertiefung einfrben
		g.setColor(ThreeDPrefs.sliderTrackColor);
		g.fillRect(draw.x+i.left, draw.y+i.top, draw.width-i.left-i.right, draw.height-i.top-i.bottom);
	}

}
