/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.threeD;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class ThreeD_TabbedPaneUI extends BasicTabbedPaneUI {
	
	public static ComponentUI createUI(JComponent c)  {
		return new ThreeD_TabbedPaneUI();
	}

	public void paint(Graphics g, JComponent c)  {
		int 			tabPlacement 	= tabPane.getTabPlacement();
		Insets 		insets 			= c.getInsets(); 
		Dimension 	size 			= c.getSize();
		int 			totalTabWidth 	= 0;
		int 			totalTabHeight 	= 0;
		
		// Paint the background for the tab area
		if ( tabPane.isOpaque() ) 
			{
			g.setColor(c.getBackground());
			switch (tabPlacement) 
				{
				case LEFT:
					totalTabWidth = calculateTabAreaWidth(tabPlacement, runCount, maxTabWidth);
					g.fillRect(insets.left, insets.top, totalTabWidth, size.height - insets.bottom - insets.top );
					break;
				case BOTTOM:
					totalTabHeight = calculateTabAreaHeight(tabPlacement, runCount, maxTabHeight);
					g.fillRect( insets.left, size.height - insets.bottom - totalTabHeight, size.width - insets.left - insets.right, totalTabHeight );
					break;
				case RIGHT:
					totalTabWidth = calculateTabAreaWidth(tabPlacement, runCount, maxTabWidth);
					g.fillRect(size.width - insets.right - totalTabWidth, insets.top, totalTabWidth, size.height - insets.top - insets.bottom);
					break;
				default: // TOP
					totalTabHeight = calculateTabAreaHeight(tabPlacement, runCount, maxTabHeight);
					g.fillRect( insets.left, insets.top, size.width - insets.right - insets.left, totalTabHeight);
				}
			}
		
		super.paint(g, c);
		if (! c.isEnabled()) ThreeDUtils.drawDisabledState(g, c);
	}

	protected void paintContentBorderBottomEdge(Graphics g, int tabPlacement, int selectedIndex, int x, int y, int w, int h)  {
		g.setColor(ThreeDPrefs.darkEdgeColor);
		
		if (tabPlacement != BOTTOM || selectedIndex < 0 || (rects[selectedIndex].y - 1 > h)) 
			{
			g.drawLine(x+1, y+h-1, x+w-1, y+h-1);
			} 
		else 
			{
			Rectangle selRect = rects[selectedIndex];
			g.drawLine(x+1, y+h-1, selRect.x, y+h-1);
			if (selRect.x + selRect.width < x + w - 2) 
				{
				g.drawLine(selRect.x + selRect.width, y+h-1, x+w-1, y+h-1);
				} 
			}
	}

	protected void paintContentBorderLeftEdge(Graphics g, int tabPlacement, int selectedIndex, int x, int y, int w, int h)  {
		g.setColor(ThreeDPrefs.lightEdgeColor); 
		
		if (tabPlacement != LEFT || selectedIndex < 0 || (rects[selectedIndex].x + rects[selectedIndex].width + 1< x)) 
			{
			g.drawLine(x, y, x, y+h-2);
			}
		else 
			{
		 	Rectangle selRect = rects[selectedIndex];
			g.drawLine(x, y, x, selRect.y);
			if (selRect.y + selRect.height < y + h - 2) 
				{
				g.drawLine(x, selRect.y + selRect.height, x, y+h+2);
				} 
			}
	}

	protected void paintContentBorderRightEdge(Graphics g, int tabPlacement, int selectedIndex, int x, int y, int w, int h)  {
		g.setColor(ThreeDPrefs.darkEdgeColor);
		
		if (tabPlacement != RIGHT || selectedIndex < 0 || rects[selectedIndex].x - 1 > w) 
			{
			g.drawLine(x+w-1, y+1, x+w-1, y+h-1);
			} 
		else 
			{
			Rectangle selRect = rects[selectedIndex];
			g.drawLine(x+w-1, y+1, x+w-1, selRect.y);
			if (selRect.y + selRect.height < y + h - 2) 
				{
				g.drawLine(x+w-1, selRect.y + selRect.height-1, x+w-1, y+h-2);
				}
			}
	}

	protected void paintContentBorderTopEdge(Graphics g, int tabPlacement, int selectedIndex, int x, int y, int w, int h)  {
		g.setColor(ThreeDPrefs.lightEdgeColor);
		
		if (tabPlacement != TOP || selectedIndex < 0 || (rects[selectedIndex].y + rects[selectedIndex].height + 1 < y)) 
			{
			g.drawLine(x, y, x+w-2, y);
			} 
		else 
			{
			Rectangle selRect = rects[selectedIndex];
			g.drawLine(x, y, selRect.x, y);
			if (selRect.x + selRect.width < x + w - 2) 
				{
				g.drawLine(selRect.x + selRect.width, y, x+w-2, y);
				} 
			}
	}

	protected void paintFocusIndicator(Graphics g, int tabPlacement, Rectangle [] rects, int tabIndex, Rectangle iconRect, Rectangle textRect, boolean isSelected)  {
		if (! isSelected) return;
		if (! tabPane.hasFocus()) return;
		
		if (textRect.width > 0 && iconRect.width > 0)
			{
			Rectangle drawRect = iconRect.union(textRect);
			g.setColor(ThreeDPrefs.focusColor);
			g.drawRect(drawRect.x - 1, drawRect.y - 1, drawRect.width + 1, drawRect.height + 1);
			}
		else if (textRect.width > 0) // Text umranden 
			{
			g.setColor(ThreeDPrefs.focusColor);
			g.drawRect(textRect.x - 1, textRect.y - 1, textRect.width + 1, textRect.height + 1);
			}
		else if (iconRect.width > 0) // Icon umranden
			{
			g.setColor(ThreeDPrefs.focusColor);
			g.drawRect(iconRect.x - 1, iconRect.y - 1, iconRect.width + 1, iconRect.height + 1);
			}
		// else System.err.println("ThreeD_TabbedPaneUI.paintFocusIndicator(): TabbedPane without text and icon!");
	}

	protected void paintTabBackground(Graphics g, int tabPlacement, int tabIndex, int x, int y, int w, int h, boolean isSelected)  {
		g.setColor(isSelected ? ThreeDPrefs.backgroundColor : ThreeDPrefs.tabbedPaneBackgroundColor);
		g.fillRect(x,y,w-1,h-1);
	}

	protected void paintTabBorder(Graphics g, int tabPlacement, int tabIndex, int x, int y, int w, int h, boolean isSelected)  {
		g.translate(x,y);
		
		w--;
		h--;
		
		switch (tabPlacement) 
			{
			case LEFT:
				if (isSelected) { w--; h--; } // BUG
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(0,0,w,0);
				g.drawLine(0,0,0,h-1);
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(1, h, w, h);
				break;
			case BOTTOM:
				if (isSelected) { y++; w--; } // BUG
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(0,0,0,h-1);
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(1, h, w, h);
				g.drawLine(w,h,w,0);
				break;
			case RIGHT:
				if (isSelected) { x++; w--; h--; } // BUG
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(0,0,w-1,0);
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(w, 1, w, h);
				g.drawLine(0, h,  w, h);
				break;
			case TOP:
				if (isSelected) { h--; w--; } // BUG
				g.setColor(ThreeDPrefs.lightEdgeColor);
				g.drawLine(0,0,0,h);
				g.drawLine(0,0,w-1,0);
				g.setColor(ThreeDPrefs.darkEdgeColor);
				g.drawLine(w, 1, w, h);
			default:
			}
		
		g.translate(-x,-y);
	}

}
